/*
 * A very basic "collection" API
 *
 * Copyright (C) 2000 Ben Martin (aka monkeyiq)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/*

	The semantics of this collection are *very* loose, its like a non ordered stack, 
	you can put stuff in, and get them back in ANY order, the only catch is that the "any"
	is decided by the ADT, not the caller.
	
	I hope to make put/push O(1) in the future, I have not checked that they both are
	at the moment.
	
	This is a great datatype to use for buckets in a hashtable, provieded that you
	use a write all then real all plan. ie. Fully populate the hashtable and then drain
	all the data back from the hashtable.

*/
#ifndef INCLUDED_GBLOB_C
#define INCLUDED_GBLOB_C 1

#include "gblob.h"

/**
 * Create a new blob, returns NULL on error, or a new blob
 */
GBlob*
g_blob_new() 
{
	GBlob*	ret = g_new0(GBlob,1);
	ret->a = 0;
	return ret;
}

/**
 * Add an entry to the blob. Returns 1 on error, 0 if ok.
 */
gint
g_blob_push( GBlob* s, gpointer data)
{
	g_assert ( s != NULL );
	
	s->a = g_slist_prepend( s->a, data);
	return 0;
}

/**
 * Get an entry from a blob, returns an entry or NULL
 */
gpointer
g_blob_pop( GBlob* s )
{
	GSList* f = NULL;
	gpointer ret = NULL;
	
	g_assert ( s != NULL );
	g_assert ( s->a != NULL );

	if( s && s->a ) {
		f = g_slist_nth(s->a,0);
		if( f ) {
			ret = f->data;
			s->a = g_slist_remove ( s->a, ret);
		}
	}
	return ret;
}


#endif /* INCLUDED_GBLOB_C */
