
/*
 * ldap gconf backend
 *
 * Copyright (C) 2000 Ben Martin (aka monkeyiq)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef INCLUDED_LDAP_BACKEND_H
#define INCLUDED_LDAP_BACKEND_H


// ldap
#include <ldap.h> 
 
// gconf
#include <gconf/gconf-backend.h>
#include <gconf/gconf-internals.h>
#include <gconf/gconf.h>

/* local */
#include "serialize.h"




struct _LDAPSource {
	GConfSource source; /* inherit from GConfSource */
	
	LDAP*   	ld;				/* Main LDAP connection */
	gchar*  	server_url;		/* URL for the LDAP server */
	gchar* 		username;		/* Username to use for simple auth */
	gchar*		password;		/* Password for simple auth */
	int 		seconds_to_cache;	/* time to cache data at client end */
	int 		max_levels_of_depth;/* how deep the LDAP directory names can ever be, security check */
	gchar*		key_base;			/* base dn for GNOME data in LDAP repository */
	serialize_as_t serialize_as;	/* How the user wants us to store string/int/list data */
	
};
typedef struct _LDAPSource LDAPSource;


/**
 * This macro can be used at the top of vtable funcitons to declare and setup 
 * local variables that point to the instance variables for this LDAPSource.
 *
 * it is important to realize that these are aliases and should be treated as
 * readonly immutable references.
 */
#define MAKE_INSTANCE_VARS( gconf_source , rv ) \
	 LDAPSource *      	ld_source = (LDAPSource*)gconf_source; \
	 LDAP 		*		ld = ld_source ? ld_source->ld : NULL; \
	\
	
#define MAKE_INSTANCE_VARS_ASSERT \
	g_return_val_if_fail( gconf_source != NULL, rv );  \
	g_return_val_if_fail( ld != NULL, rv );  

// keys that are used for each LDAP entry for attribute names
#define LDAP_GCONF_TYPE_KEY 			"type"	
#define LDAP_GCONF_TYPE_MARSHALED 		"type_marshaled"	/* For single shot marshaled values */
#define LDAP_GCONF_CAR_TYPE_KEY 		"type_car"	
#define LDAP_GCONF_CDR_TYPE_KEY 		"type_cdr"	
#define LDAP_GCONF_VALUE_KEY 			"value"	
#define LDAP_GCONF_LIST_VALUE_KEY 		"values"
#define LDAP_GCONF_MARSHALED_VALUE_KEY 	"value_marshaled"
#define LDAP_GCONF_CAR_VALUE_KEY 		"value_car"	
#define LDAP_GCONF_CDR_VALUE_KEY 		"value_cdr"	
#define LDAP_GCONF_OBJECTCLASS_KEY 		"objectClass"		/* For better LDAP schemas */
#define LDAP_GCONF_OBJECTCLASS 			"externalObject"	/* For better LDAP schemas */
#define LDAP_GCONF_META_CREATORNAME_KEY "creatorsname"
#define LDAP_GCONF_META_MODIFIERSNAME_KEY "modifiersname"
#define LDAP_GCONF_META_MODIFYTIMESTAMP "modifytimestamp"
#define LDAP_GCONF_META_SCHEMA_POSTFIX "_meta_data"
#define LDAP_GCONF_META_SCHEMA_KEY 		"meta_data"

// Min amount of feilds that the user must supply in backend data.
#define BACKEND_ARGS_MIN				6
#define BACKEND_ARG_USERNAME 			0 // offsets into the backend data for each item
#define BACKEND_ARG_PASSWORD 			1
#define BACKEND_ARG_SSL 				2
#define BACKEND_ARG_TREEBASE			3
#define BACKEND_ARG_SECONDS_TO_CACHE	4
#define BACKEND_ARG_MAX_LEVELS_DEPTH	5
#define BACKEND_ARG_SERVER_LOCATIONS	6

#endif /* ifndef INCLUDED_LDAP_BACKEND_H */ 
