/*
 * ldap gconf backend
 *
 * Copyright (C) 2000 Ben Martin (aka monkeyiq)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#define MAX_SETTABLE_ATTRS 1000

#include <ldap.h>
#include <glib.h>


/**
 * This function creates a new LDAPMod abstraction for use with 
 * ldap_gconf_ldapmodvec_addmod()
 * ldap_gconf_ldapmodvec_new()
 * ldap_gconf_ldapmodvec_toArray()
 *
 * By using these functions the LDAP C API is a tad less hairy.
 *
 */
LDAPMod *
ldap_gconf_ldapmod_new();

/**
 * Add a new mod to the vector of changes that you are creating for use with
 * ldap_modify()
 *
 * a The array of changes see:
 * ldap_gconf_ldapmodvec_new()
 * ldap_gconf_ldapmodvec_toArray()
 *
 * mod_op, mod_type are exactly like in the RFC doco for the C LDAP API.
 * the "..." params are gchar*, basically the string vector part of the C LDAP API.
 *
 * mod_type and the ... params are copied by this function.
 *
 *
 */
void
ldap_gconf_ldapmodvec_addmod( GPtrArray* a, int mod_op, char *mod_type, ...);

/**
 * Add a new mod to the vector of changes that you are creating for use with
 * ldap_modify()
 *
 * a The array of changes see:
 * ldap_gconf_ldapmodvec_new()
 * ldap_gconf_ldapmodvec_toArray()
 *
 * mod_op, mod_type are exactly like in the RFC doco for the C LDAP API.
 * modv is copied by this function.
 *
 * mod_type and the ... params are copied by this function.
 *
 *
 */
void
ldap_gconf_ldapmodvec_addmodv( GPtrArray* a, int mod_op, char *mod_type, char** modv);

/**
 *  Create a new vector of modifications to make with ldap_modify()
 * see also
 * ldap_gconf_ldapmodvec_addmod()
 * ldap_gconf_ldapmodvec_new()
 * ldap_gconf_ldapmodvec_toArray()
 */
GPtrArray* 
ldap_gconf_ldapmodvec_new();

/**
 *  Obtain data in the required format of LDAPMods for ldap_modify()
 * see also
 * ldap_gconf_ldapmodvec_addmod()
 * ldap_gconf_ldapmodvec_new()
 */
LDAPMod** ldap_gconf_ldapmodvec_toArray( GPtrArray* a );

/**
 * Free a vector of modifications
 *
 *
 * see also
 * ldap_gconf_ldapmodvec_addmod()
 * ldap_gconf_ldapmodvec_new()
 */
void
ldap_gconf_ldapmodvec_free( GPtrArray* a );
