
/*
 * ldap gconf backend
 *
 * Copyright (C) 2000 Ben Martin (aka monkeyiq)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// gconf
#include <gconf/gconf-backend.h>
#include <gconf/gconf-internals.h>
#include <gconf/gconf.h>

// glib
#include <glib.h>

#include <stdio.h>

#include "ldap-backend.c" // hack!

void 
tryIO( 
	GConfSource* source,
	GError** err,
	GConfValue* value,
	gchar* key
	)
{
	GConfValue* value2 = NULL;

	printf("tryIO\n");

	printf("tryIO: Setting value with ldap_gconf_set_value...\n");	
	ldap_gconf_set_value( source, key, value, err );	
	
	printf("tryIO: Getting value with ldap_gconf_query_value...\n");	
	value2 = ldap_gconf_query_value( source, key, NULL, NULL, err );
	if(value2) {
		printf("tryIO: Read back a value: %s\n",	gconf_value_to_string(value2));
	}	
}

void 
tryIntIO( 
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* key = "/test/one/two/three/IntTest";

	printf("tryIntIO\n");
	printf("tryIntIO: Create a new GConfValue that holds an int value=69...\n");
	value = gconf_value_new_from_string(GCONF_VALUE_INT, "69", err);
	printf("tryIntIO: calling tryIO with value and key:%s...\n",key);	
	tryIO( source, err, value, key );
}

void 
tryStrIO( 
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* key = "/test/one/two/three/StrTest";

	printf("tryStrIO\n");
	printf("tryStrIO: Create a new GConfValue that holds a str value=As good as it gets...\n");
	value = gconf_value_new_from_string(GCONF_VALUE_STRING, "As good as it gets", err);
	printf("tryStrIO: calling tryIO with value and key:%s...\n",key);	
	tryIO( source, err, value, key );
}

void 
tryFloatIO( 
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* key = "/test/one/two/three/FloatTest";

	printf("tryFloatIO\n");
	printf("tryFloatIO: Create a new GConfValue that holds a value=3.14...\n");
	value = gconf_value_new_from_string(GCONF_VALUE_FLOAT, "3.14", err);
	printf("tryFloatIO: calling tryIO with value and key:%s...\n",key);	
	tryIO( source, err, value, key );
}

void 
tryBoolIO( 
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* keyT = "/test/one/two/three/BoolTestTrue";
	gchar* keyF = "/test/one/two/three/BoolTestFalse";

	printf("tryFloatIO\n");
	printf("tryFloatIO: Create a new GConfValue that holds a value=true...\n");
	value = gconf_value_new_from_string(GCONF_VALUE_BOOL, "true", err);
	printf("tryFloatIO: calling tryIO with value and key:%s...\n",keyT);	
	tryIO( source, err, value, keyT );

	printf("tryFloatIO\n");
	printf("tryFloatIO: Create a new GConfValue that holds a value=false...\n");
	value = gconf_value_new_from_string(GCONF_VALUE_BOOL, "false", err);
	printf("tryFloatIO: calling tryIO with value and key:%s...\n",keyF);	
	tryIO( source, err, value, keyF );
}


void 
tryPairIO( 
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* key = "/test/one/two/three/PairTest";

	printf("tryPairIO\n");
	printf("tryPairIO: Create a new GConfValue that holds a value=(left,right)...\n");
//	value = gconf_value_new_pair_from_string(GCONF_VALUE_STRING, GCONF_VALUE_STRING, "(left,right)",err);
	value = gconf_value_new(GCONF_VALUE_PAIR);
  	gconf_value_set_car_nocopy(value, gconf_value_new_from_string(GCONF_VALUE_STRING, "left", err));
  	gconf_value_set_cdr_nocopy(value, gconf_value_new_from_string(GCONF_VALUE_STRING, "right", err));
	if(!value) {
		printf("tryPairIO: failed to create pair value (not backend specific)\n");
		return;
	}
	printf("tryPairIO: calling tryIO with value and key:%s...\n",key);	
	tryIO( source, err, value, key );
}

void 
tryListIO( 
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* key = "/test/one/two/three/ListTest";
	gchar* vals[] = {"one","two","three","four","V","VI","VII","VIII","EOU",NULL};
	gchar** vals2 = NULL;

	printf("tryListIO\n");
	printf("tryListIO: Create a new GConfValue that holds a value=[one,two,three,four,V,VI,VII,VIII,EOU]...\n");
//	value = gconf_value_new_list_from_string(GCONF_VALUE_STRING, "[one,two,three,four,V,VI,VII,VIII,EOU]",err);
	value = gconf_value_set_list_strv_nocopy( g_strv_copy(vals));
	printf("tryListIO: created list: %s \n", gconf_value_to_string(value));
	vals2 = __gconf_list_to_strv( value );	
		
	if(!value) {
		printf("tryListIO: failed to create list value (not backend specific)\n");
		return;
	}
	printf("tryListIO: calling tryIO with value and key:%s...\n",key);	
	tryIO( source, err, value, key );
}

void
Try_all_entries_f(
	gpointer data,
	gpointer user_data
	)
{
	GConfValue* v = (GConfValue*)data;
	printf("Try_all_entries_f \n");
	if(v) {
		printf("Try_all_entries_f val: %s\n", gconf_value_to_string(v));
	}
}

void 
Try_all_entries(
	GConfSource* source,
	GError** err)
{
	gchar* key = "/test/one/two/three";
	
	static GSList* list;
	list = 
		ldap_gconf_all_entries    (source,
                key,
                NULL,
                err);
	
	g_slist_foreach( list, Try_all_entries_f, NULL );
}

void
Try_all_subdirs_f(
	gpointer data,
	gpointer user_data
	)
{
	gchar* v = (gchar*)data;
	printf("Try_all_subdirs_f \n");
	if(v) {
		printf("Try_all_subdirs_f val: %s\n", v);
	}
}

void 
Try_all_subdirs(
	GConfSource* source,
	GError** err)
{
	gchar* key = "";
	
	static GSList* list;
	list = 
		ldap_gconf_all_subdirs
				(source,
                key,
                err);
	
	g_slist_foreach( list, Try_all_subdirs_f, NULL );
}

void
Try_dir_exists(
	GConfSource* source,
	GError** err)
{
	gchar* key_ok = "/test/one/two/three";
	gchar* key_fail = "/this/dir/should/not/exist/file_not_exist";
	printf("Try_dir_exists key:%s should be true ==> %d\n",
		key_ok,
		ldap_gconf_dir_exists( source, key_ok, err )
		);
	printf("Try_dir_exists key:%s should be false ==> %d\n",
		key_fail,
		ldap_gconf_dir_exists( source, key_fail, err )
		);
}

void
Try_unset_value(
	GConfSource* source,
	GError** err)
{
	GConfValue* value = NULL;
	gchar* key = "/test/one/two/three/unset_value_test";

	printf("Try_unset_value\n");
	printf("Try_unset_value: Create a new GConfValue that holds a str value=kitten...\n");
	value = gconf_value_new_from_string(GCONF_VALUE_STRING, "kitten", err);
	printf("Try_unset_value: calling tryIO with value and key:%s...\n",key);	
	tryIO( source, err, value, key );
	
	ldap_gconf_unset_value( source, key, NULL, err );
	
}

void
Try_remove_dir(
	GConfSource* source,
	GError** err)
{
	gchar* key = "/test";
	ldap_gconf_remove_dir( source, key, err );
}


void 
try_set_meta(
	GConfSource* source,
	GError** err)
{
	gchar* key = "/test/meta/key";
	gchar* schema = "/test/metadata/key";
	ldap_gconf_set_schema( source, key, schema, err );
}

void 
try_get_meta(
	GConfSource* source,
	GError** err)
{
	gchar* key = "/test/meta/key";
	GConfMetaInfo* mi = ldap_gconf_query_metainfo( source, key, err );
	
	if(mi) {
		printf("try_get_meta: schema: %s mod_user:%s mod_time:%s\n",
			mi->schema, mi->mod_user, ctime((time_t*)(&mi->mod_time)));
	}
	
}

int main( int argc, char** argv ) {
	GError* err;
	GConfSource* source;
	const gchar* backend_args 
		= "ldap:readwrite: -p secret -l 'cn=Manager, dc=monkeyiq, dc=com' -s 600 -d 100 -k 'dc=gnome, dc=monkeyiq, dc=com'  localhost";
	/** 
		"ldap:readwrite:dummy 
	1		-p secret 
	0		-l 'cn=Manager, dc=monkeyiq, dc=com' 
			-s 600 -d 100 
	4		-k 'dc=gnome, dc=monkeyiq, dc=com'  
			localhost";
	*/

	
	printf("Starting...\n");
	
	printf("Get vtable...\n");
	gconf_backend_get_vtable();

	printf("Init module...\n");
	g_module_check_init(NULL);
	
	printf("Resolve address...\n");	
	source = ldap_gconf_resolve_address(backend_args,	&err);
	
	tryIntIO( source, &err );
	tryStrIO( source, &err );
	tryFloatIO( source, &err );
	tryBoolIO( source, &err );
	tryPairIO( source, &err );
//	tryListIO( source, &err );
	
	
//	Try_all_entries( source, &err );
//	Try_all_subdirs( source, &err );

	//Try_dir_exists( source, &err );
	//Try_unset_value( source, &err );

//	tryStrIO( source, &err );
//	Try_remove_dir( source, &err );

//	tryPairIO( source, &err );

//	try_set_meta( source, &err );
//	try_get_meta( source, &err );
	
	printf("Free source. (lol)\n");	
	ldap_gconf_destroy_source( source );
	
	printf("Done.\n");
	
	return 0;
}
